@@ -1125,14 +1125,14 @@ MrdbCursor_execute_text(MrdbCursor *self, PyObject *stmt)
 {
     int rc;
     MYSQL *db;
-    const char *statement;
+    char *statement;
     size_t statement_len= 0;
 
     MARIADB_CHECK_CONNECTION(self->connection, NULL);
 
     if (Py_TYPE(stmt) == &PyUnicode_Type)
     {
-        statement = PyUnicode_AsUTF8AndSize(stmt, (Py_ssize_t *)&statement_len);
+        statement = (char *)PyUnicode_AsUTF8AndSize(stmt, (Py_ssize_t *)&statement_len);
     } else if (Py_TYPE(stmt) == &PyBytes_Type)
     {
         PyBytes_AsStringAndSize(stmt, &statement, (Py_ssize_t *)&statement_len);
@@ -1225,7 +1225,10 @@ MrdbCursor_execute_bulk(MrdbCursor *self)
     mysql_stmt_bind_param(self->stmt, self->params);
 
     if (!(buf= self->connection->mysql->methods->db_execute_generate_request(self->stmt, &buflen, 1)))
+    {
+        mariadb_throw_exception(self->stmt, NULL, 1, NULL);
         goto error;
+    }
 
     if ((rc= Mrdb_execute_direct(self, self->parseinfo.statement, self->parseinfo.statement_len)))
     {
