@@ -40,6 +40,24 @@ class TestCursor(unittest.TestCase):
         cursor.close()
         del cursor
 
+    def test_conpy283(self):
+        conn= create_connection()
+        cursor= conn.cursor(dictionary=True)
+        self.assertEqual(cursor._resulttype, 2)
+        cursor.execute("select 1 as A union SELECT 2 as A")
+        row= cursor.fetchone()
+        self.assertEqual(row, {'A' : 1})
+        self.assertEqual(cursor._resulttype, 2)
+        cursor.scroll(-1)
+        self.assertEqual(cursor._resulttype, 2)
+        row= cursor.fetchone()
+        self.assertEqual(row, {'A' : 1})
+        row= cursor.fetchone()
+        self.assertEqual(row, {'A' : 2})
+        self.assertEqual(cursor._resulttype, 2)
+        cursor.close()
+        conn.close()
+
     def test_date(self):
         v = self.connection.server_version
         i = self.connection.server_info.lower()
@@ -259,35 +277,12 @@ class TestCursor(unittest.TestCase):
         self.assertEqual(metadata["type"][3], FIELD_TYPE.STRING)
         self.assertEqual(metadata["ext_type_or_format"][4], EXT_FIELD_TYPE.POINT)
         self.assertEqual(metadata["type"][4], FIELD_TYPE.GEOMETRY)
-        cursor.execute("SELECT a,b,c,d,e FROM t1 WHERE 1=?", (1,))
-        metadata= cursor.metadata
-        self.assertEqual(metadata["ext_type_or_format"][0], EXT_FIELD_TYPE.JSON)
-        self.assertEqual(metadata["type"][0], FIELD_TYPE.BLOB)
-        self.assertEqual(metadata["ext_type_or_format"][1], EXT_FIELD_TYPE.UUID)
-        self.assertEqual(metadata["type"][1], FIELD_TYPE.STRING)
-        self.assertEqual(metadata["ext_type_or_format"][2], EXT_FIELD_TYPE.INET4)
-        self.assertEqual(metadata["type"][2], FIELD_TYPE.STRING)
-        self.assertEqual(metadata["ext_type_or_format"][3], EXT_FIELD_TYPE.INET6)
-        self.assertEqual(metadata["type"][3], FIELD_TYPE.STRING)
-        self.assertEqual(metadata["ext_type_or_format"][4], EXT_FIELD_TYPE.POINT)
-        self.assertEqual(metadata["type"][4], FIELD_TYPE.GEOMETRY)
 
         cursor.close()
 
     def test_xfield_types(self):
-        cursor = self.connection.cursor()
-        cursor.execute("SELECT a,b,c,d,e FROM t1")
-        description= cursor.description
-        self.assertEqual(12, len(description[0]))
-        self.assertEqual(description[0][11], EXT_FIELD_TYPE.JSON)
-        cursor.execute("SELECT a,b,c,d,e FROM t1 WHERE 1=?", (1,))
-        description= cursor.description
-        self.assertEqual(12, len(description[0]))
-        self.assertEqual(description[0][11], EXT_FIELD_TYPE.JSON)
-        cursor.close()
-        connection.close()
-
-    def test_xfield_types(self):
+        if is_maxscale():
+            self.skipTest("Test doesn't work with maxscale")
         cursor = self.connection.cursor()
         fieldinfo = mariadb.fieldinfo()
         cursor.execute("CREATE TEMPORARY TABLE test_xfield_types ("
@@ -1596,6 +1591,22 @@ class TestCursor(unittest.TestCase):
         cursor.close()
         connection.close()
 
+    def test_conpy291(self):
+        if is_mysql:
+            self.skipTest("Skip (MySQL doesn't support batch/indicators)")
+        connection = create_connection()
+        cursor = connection.cursor()
+
+        cursor.execute("DROP TABLE IF EXISTS t1")
+        cursor.execute("CREATE TABLE t1 (a int, b int, c varchar(100), d int)")
+
+        data= [(1, INDICATOR.NULL, "foo", INDICATOR.NULL),
+               (2, 3, "foo", 5)]
+        cursor.executemany("INSERT INTO t1 VALUES (?,?,?,?)", data)
+        self.assertEqual(cursor.rowcount, 2)
+        cursor.execute("DROP TABLE IF EXISTS t1")
+               
+
     def test_conpy276(self):
         connection = create_connection()
         cursor = connection.cursor()
@@ -1613,6 +1624,25 @@ class TestCursor(unittest.TestCase):
         self.assertEqual(cursor.rowcount, 4)
         cursor.close()
 
+    def test_conpy289(self):
+        if is_mysql:
+            self.skipTest("Skip (MySQL doesn't support batch)")
+        with create_connection() as conn:
+            cursor= conn.cursor()
+            cursor.execute("CREATE OR REPLACE TABLE t289 (a bigint unsigned,"\
+                           "b bigint unsigned, c bigint unsigned,"\
+                           "d bigint unsigned)")
+
+            data= [(0, 9232974212090577672, 11529232914765115761, 58),
+                   (0, 13837380911034793984, 11529233872889843812, 0),
+                   (0, 0, 2351636844950589488, 9999)]
+
+            cursor.executemany("insert into t289 values (?,?,?,?)", data)
+
+            cursor.execute("SELECT * FROM t289")
+            rows= cursor.fetchall()
+            self.assertEqual(rows, data)
+
     def test_conpy91(self):
         with create_connection() as connection:
             with connection.cursor() as cursor:
